//
// Sample to show multitexturing in OpenGL
//
// Author: Alex V. Boreskoff <alexboreskoff@mtu-net.ru>, <steps3d@narod.ru>
//

#include	<gl/glut.h>
#include	<gl/glaux.h>
#include	"../glext.h"
#include	<stdio.h>
#include	<string.h>
#include	<ctype.h>

float	 angle  = 0;
float	 angle2 = 0;
unsigned texture1;
unsigned texture2;

unsigned	loadTexture ( const char * texName )
{
								// load texture
	AUX_RGBImageRec * texture = auxDIBImageLoad ( texName );
	unsigned		  textureId;

	glGenTextures   ( 1, &textureId );
	glBindTexture   ( GL_TEXTURE_2D, textureId );
	glPixelStorei   ( GL_UNPACK_ALIGNMENT, 1 );							// set 1-byte alignment

	glTexParameteri ( GL_TEXTURE_2D,  GL_TEXTURE_WRAP_S, GL_REPEAT );	// set texture to repeat mode
	glTexParameteri ( GL_TEXTURE_2D,  GL_TEXTURE_WRAP_T, GL_REPEAT );

	gluBuild2DMipmaps ( GL_TEXTURE_2D, GL_RGB, 
                        texture -> sizeX, texture -> sizeY, 
                        GL_RGB, GL_UNSIGNED_BYTE, 
                        texture -> data );

	glTexParameteri   ( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );	
	glTexParameteri   ( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );

	return textureId;
}

bool	isExtensionSupported ( const char * ext )
{
	const char * extensions = (const char *)glGetString ( GL_EXTENSIONS );
	const char * start      = extensions;
	const char * ptr;

	while ( ( ptr = strstr ( start, ext ) ) != NULL )
	{
											// we've found, ensure name is exactly ext
		const char * end = ptr + strlen ( ext );

		if ( isspace ( *end ) || *end == '\0' )
			return true;

		start = end;
	}

	return false;
}

PFNGLACTIVETEXTUREARBPROC			glActiveTextureARB       = NULL;
PFNGLCLIENTACTIVETEXTUREARBPROC 	glClientActiveTextureARB = NULL;
PFNGLMULTITEXCOORD1FARBPROC			glMultiTexCoord1f        = NULL;
PFNGLMULTITEXCOORD1FVARBPROC		glMultiTexCoord1fv       = NULL;
PFNGLMULTITEXCOORD2FARBPROC			glMultiTexCoord2f        = NULL;
PFNGLMULTITEXCOORD2FVARBPROC		glMultiTexCoord2fv       = NULL;

bool	initMultitexture ()
{
	glActiveTextureARB       = (PFNGLACTIVETEXTUREARBPROC)       wglGetProcAddress ( "glActiveTextureARB"       );
	glClientActiveTextureARB = (PFNGLCLIENTACTIVETEXTUREARBPROC) wglGetProcAddress ( "glClientActiveTextureARB" );
	glMultiTexCoord1f        = (PFNGLMULTITEXCOORD1FARBPROC)     wglGetProcAddress ( "glMultiTexCoord1fARB"     );
	glMultiTexCoord1fv       = (PFNGLMULTITEXCOORD1FVARBPROC)    wglGetProcAddress ( "glMultiTexCoord1fvARB"    );
	glMultiTexCoord2f        = (PFNGLMULTITEXCOORD2FARBPROC)     wglGetProcAddress ( "glMultiTexCoord2fARB"     );
	glMultiTexCoord2fv       = (PFNGLMULTITEXCOORD2FVARBPROC)    wglGetProcAddress ( "glMultiTexCoord2fvARB"    );

	return glActiveTextureARB       != NULL &&
		   glClientActiveTextureARB != NULL &&
		   glMultiTexCoord1f        != NULL &&
		   glMultiTexCoord1fv       != NULL &&
		   glMultiTexCoord2f        != NULL &&
		   glMultiTexCoord2fv       != NULL;   
}

void init ()
{
	glClearColor ( 0, 0, 0, 1 );
	glEnable     ( GL_DEPTH_TEST );

	glActiveTextureARB ( GL_TEXTURE0_ARB );
	glEnable           ( GL_TEXTURE_2D );
	glBindTexture      ( GL_TEXTURE_2D, texture1 );
	glTexEnvi          ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );

	glActiveTextureARB ( GL_TEXTURE1_ARB );
	glEnable           ( GL_TEXTURE_2D );
    glEnable           ( GL_TEXTURE_GEN_S );
    glEnable           ( GL_TEXTURE_GEN_T );
    glTexGeni          ( GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP );
    glTexGeni          ( GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP );
	glBindTexture      ( GL_TEXTURE_2D, texture2 );
	glTexEnvi          ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
}

void display ()
{
	glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix    ();
	glTranslatef    ( 2, 2, 2 );
	glRotatef       ( angle,  1, 1, 0 );
	glRotatef       ( angle2, 0, 1, 1 );
	glutSolidTeapot ( 1 );
	glPopMatrix     ();

	glutSwapBuffers ();
}

void reshape ( int w, int h ) 
{
   glViewport     ( 0, 0, (GLsizei)w, (GLsizei)h );
   glMatrixMode   ( GL_PROJECTION );
   glLoadIdentity ();
   gluPerspective ( 60.0, (GLfloat)w/(GLfloat)h, 1.0, 60.0 );
   glMatrixMode   ( GL_MODELVIEW );
   glLoadIdentity ();
   gluLookAt      ( 0.0, 0.0, 0.0,		// eye
					5.0, 5.0, 5.0,		// center
					0.0, 1.0, 0.0 );
}

void key ( unsigned char key, int x, int y )
{
  if ( key == 27 || key == 'q' || key == 'Q' )		//quit requested
    exit ( 0 );
}

void	animate ()
{
	angle  = 0.04f * glutGet ( GLUT_ELAPSED_TIME );
	angle2 = 0.01f * glutGet ( GLUT_ELAPSED_TIME );

	glutPostRedisplay ();
}

int main ( int argc, char * argv [] )
{
								//initialise glut
	glutInit            ( &argc, argv );
	glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );   
	glutInitWindowSize  ( 400, 400 );


								// create window
	int	glWin = glutCreateWindow ( "OpenGL Multitexture example 2" );

								// register handlers
	glutDisplayFunc  ( display );
	glutReshapeFunc  ( reshape );
    glutKeyboardFunc ( key );
	glutIdleFunc     ( animate );

	const char * vendor    = (const char *)glGetString ( GL_VENDOR     );
	const char * renderer  = (const char *)glGetString ( GL_RENDERER   );
	const char * version   = (const char *)glGetString ( GL_VERSION    );
	const char * extension = (const char *)glGetString ( GL_EXTENSIONS );

	printf ( "Vendor:   %s\nRenderer: %s\nVersion:  %s\n", vendor, renderer, version );

	if ( !isExtensionSupported ( "GL_ARB_multitexture" ) )
	{
		printf ( "ARB_multitexture NOT supported.\n" );

		return 1;
	}

	int	maxTextureUnits;

	glGetIntegerv ( GL_MAX_TEXTURE_UNITS_ARB, &maxTextureUnits );

	printf ( "ARB_multitexture supported.\nMax texture units %d.\n", maxTextureUnits );

	if ( initMultitexture () )
		printf ( "initMultitexture successful.\n" );
	else
	{
		printf ( "initMultitexture FAILED.\n" );

		return 2;
	}

	texture1 = loadTexture ( "..\\Textures\\block.bmp" );
	texture2 = loadTexture ( "..\\Textures\\stars.bmp" );

	init ();

	glutMainLoop ();

	return 0;
}
